package classserver;

import java.io.*;
import java.net.*;

/**
 * Title:        Einfacher Class Server
 * Description:  Class Server zum Herunterladen und Verschieben von Class Dateien
 * Funktioniert wie ein HTTP Server
 * Copyright:    Copyright (c) J.M.Joller
 * Company:      Joller-Voss
 * @author J.M.Joller
 * @version 1.0
 */

/**
 * Der ClassFileServer implementiert einen ClassServer welcher
 * Class Dateien lesen kann.
 */
public class ClassFileServer extends ClassServer {

    private String classpath;

    private static int DefaultServerPort = 2001;

    /**
     * Konstruktion eines ClassFileServer.
     *
     * @param classpath =Classpath wo die Klassendateien sind
     */
    public ClassFileServer(int port, String classpath) throws IOException
    {
	super(port);
	this.classpath = classpath;
    }

    /**
     * Das Bytearray enthlt die Class Datei
     * ".class" fehlt
     *
     * @return Bytecode der Klasse
     * @exception ClassNotFoundException falls die Klasse nicht gefunden wurde
     */
    public byte[] getBytes(String path)
	throws IOException, ClassNotFoundException
    {
	System.out.println("Lesen des Pfades: " + path);
	File f = new File(classpath + File.separator +
			  path.replace('.', File.separatorChar) + ".class");
	int length = (int)(f.length());
	if (length == 0) {
	    throw new IOException("Dateilaenge ist 0: " + path);
	} else {
	    FileInputStream fin = new FileInputStream(f);
	    DataInputStream in = new DataInputStream(fin);

	    byte[] bytecodes = new byte[length];
	    in.readFully(bytecodes);
	    return bytecodes;
	}
    }

    /**
     * Main Methode des Class Servers
     * Die Methode akzeptiert zwei Argumente:
     * Prot und Root des Classpath
     *
     * <code>   java ClassFileServer <port> <classpath>
     * </code><br><br>
     *
     * Codebase sollte ein einfache URL sein, inklusive Port
     *
     * <code>   java -Djava.rmi.server.codebase=http://zaphod:2001/ RMIServer
     * </code> <br><br>
     *
     * Falls Sie in den RMI Klassen einen ClassServer kreieren wollen,
     * mssen Sie einfach eine Instanz der Klasse bilden.
     *
     * <code>   new ClassFileServer(port, classpath);
     * </code>
     */
    public static void main(String args[])
    {
	int port = DefaultServerPort;
	String classpath = "";

	if (args.length >= 1) {
	    port = Integer.parseInt(args[0]);
	}

	if (args.length >= 2) {
	    classpath = args[1];
	}

	try {
            System.out.println("[ClassFileserver]main() Port = "+port+"; CLASSPATH= "+classpath);
            try {
              InetAddress addresse = InetAddress.getLocalHost();
              System.out.println("\nLocalHost\tAdresse: "+addresse);
            } catch (UnknownHostException e) {
              System.out.println("Die Adresse vom LocalHost konnte nicht gefunden werden!");
            }

	    new ClassFileServer(port, classpath);
	} catch (IOException e) {
	    System.out.println("Der Start des ClassServers schlug fehl: " +
			       e.getMessage());
	    e.printStackTrace();
	}
    }
}
